
var objectHero = require('object-hero');

function Entity(props) {
	objectHero.assignDeep(this, [props]);
	if (this.id || this.id === 0) {
		validateId(this);
	} else {
		assignUniqueId(this);
	}
}

Object.defineProperty(Entity.prototype, 'id', {
	get: function() { return this._id; },
	set: function(id) { setId(this, id); }
});

Entity.createClass = new EntityFactoryFactory(Entity);

function EntityFactoryFactory(BaseEntity) {

	function entityFactory() {
		var Constructor = function() {};
		var props = {};
		var name = 'Entity';

		Array.prototype.forEach.call(arguments, function(arg) {
			if (typeof arg === 'string') {
				name = arg;
			} else if (typeof arg === 'function') {
				Constructor = arg;
			} else if (typeof arg === 'object') {
				props = arg;
			}
		});

		Constructor._name = name;
		props = objectHero.clone(props);
		delete props._id;

		var EntityWrap = function() {
			var entity = new BaseEntity();
			objectHero.assignDeep(entity, [props]);
			entity.constructor = Constructor;
			Constructor.apply(entity, arguments);
			return entity;
		};
		EntityWrap.createClass = new EntityFactoryFactory(EntityWrap);

		return EntityWrap;
	}

	return entityFactory;
};

var entities = {};

function validateId(entity) {
	if (entities[entity.id] && entities[entity.id] !== entity) {
		console.error('entity id conflict:', entity, entities[entity.id]);
		throw new Error('entity id conflict, details in console');
	}
}

function assignUniqueId(entity) {
	var id = (entity.constructor.name || entity.constructor._name || 'Entity') + '-';
	do {
		id += Math.floor(10 * Math.random());
	} while (entities[id]);
	entities[id] = entity;
	entity._id = id;
}

function setId(entity, id) {
	if (entities[id]) {
		throw new Error('we already have an entity with id: ' + id);
	}
	delete entities[entity.id];
	entities[id] = entity;
	entity._id = id;
}

function validateEntityId(id) {
	return !entities[id];
}

module.exports = Entity;
