
var Entity = require('./Entity.js');

function System(props) {
	this._systems = [];
	this._entityComponents = {};
	Object.assign(this, props);
}

System.create = function(props) {
	return new System(props);
};

Object.defineProperty(System.prototype, 'entities', {
	get: function() {
		return this._entities || (this._parent ? this._parent.entities : null);
	},
	set: function(entities) {
		this._entities = entities;
	}
});

System.prototype.addSystem = function(system) {
	if (!(system instanceof System)) {
		system = new System(system);
	}
	system.entities = system.entities || this.entities;
	system._parent = this;
	this._systems.push(system);
	return system;
};

System.prototype.addEntity = function(entity) {
	if (!this.entities) {
		throw new Error('no entities set on system');
	}
	if (!(entity instanceof Entity)) {
		entity = new Entity(entity);
	}
	if (this.entities[entity.id]) {
		throw new Error('Entity with id already exists: ' + entity.id);
	}
	this.entities[entity.id] = entity;
	return entity;
};

System.prototype.getEntity = function(id) {
	if (this.entities) {
		return this.entities[id];
	}
};

System.prototype.eachEntity = function(callback) {
	if (this.entities) {
		Object.keys(this.entities).forEach(function(key) {
			var entity = this.entities[key];
			var component = this.getEntitySystemComponent(entity)
			callback(entity, component);
		}.bind(this));
	}
};

System.prototype.countEntities = function() {
	return Object.keys(this.entities).length;
};

System.prototype.eachSystem = function(callback) {
	var systems = this._systems || [];
	systems.forEach(function(system) {
		callback(system);
	}.bind(this));
};

System.prototype.tickAll = function(ms) {
	this.tickSystems(ms);
	this.tick(ms);
};

System.prototype.tickSystems = function(ms) {
	this.eachSystem(function(system) {
		system.tickAll(ms);
	});
};

System.prototype.tick = function(ms) {
};

System.prototype.getEntitySystemComponent = function(entity) {
	if (!this._entityComponents[entity.id]) {
		this._entityComponents[entity.id] = {};
	}
	return this._entityComponents[entity.id];
};

module.exports = System;
